/*

	Class: BestTimesDialog
	Description: Dialog which displays the list of the highest scores.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

import java.awt.*;
import java.io.*;

final class BestTimesDialog extends Dialog {
	private final static int TYPES = 3;

	private final static String TYPE[] = {"Beginner", "Intermediate", "Expert"};
	private final static String DEFAULT_NAME = "Anonymous";
	private final static int DEFAULT_TIME = 999;

	private int time[] = new int[TYPES];
	private String name[] = new String[TYPES];

	private Label timeLabel[] = new Label[TYPES];
	private Label nameLabel[] = new Label[TYPES];

	private NewBestTimeDialog newBestTimeDialog;

	public BestTimesDialog(Frame parent) {

		super(parent,"Best Times", true);

		setLayout(new BorderLayout());
		setFont(new Font("Dialog", Font.PLAIN, 11));
		setResizable(false);

		// Set up the dialog content

		Panel panel, subpanel;
		GridBagLayout layout;
		GridBagConstraints  constraints;
		Label label;
		Button button;

		layout = new GridBagLayout();
		setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.BOTH;
		constraints.insets = new Insets(10, 15, 10, 15);

		panel = new Panel();
		layout.setConstraints(panel, constraints);
		add(panel);
		panel.setLayout(new BorderLayout(15, 15));

		subpanel = new Panel();
		panel.add("Center", subpanel);

		layout = new GridBagLayout();
		subpanel.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.NONE;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.weightx = 1.0;
		constraints.weighty = 1.0;

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.gridx = 0;
		constraints.gridy = 0;

		label = new Label("Fastest Mine Sweepers:", Label.LEFT);
		layout.setConstraints(label, constraints);
		subpanel.add(label);

		int i;

		constraints.gridx = 0;
		constraints.gridwidth = 1;
		constraints.insets = new Insets(5, 0, 0, 20);

		for(i = 0; i < TYPES; i++) {

			constraints.gridy = (i+1);

			label = new Label(TYPE[i] + ":", Label.LEFT);
			layout.setConstraints(label, constraints);
			subpanel.add(label);
		}

		constraints.gridx = 1;

		for(i = 0; i < TYPES; i++) {

			constraints.gridy = (i+1);

			timeLabel[i] = new Label("", Label.LEFT);
			layout.setConstraints(timeLabel[i], constraints);
			subpanel.add(timeLabel[i]);
		}

		constraints.gridx = 2;
		constraints.insets = new Insets(5, 0, 0, 40);

		for(i = 0; i < TYPES; i++) {

			constraints.gridy = (i+1);

			nameLabel[i] = new Label("", Label.LEFT);
			layout.setConstraints(nameLabel[i], constraints);
			subpanel.add(nameLabel[i]);

		}

		// Load the high scores

		loadTimes();

		// Add the dialog buttons

		subpanel = new Panel();
		panel.add("South", subpanel);

		layout = new GridBagLayout();
		subpanel.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.anchor = GridBagConstraints.EAST;
		constraints.insets = new Insets(0, 10, 0, 0);
		constraints.weightx = 1.0;
		constraints.ipady = 2;

		button = new Button("Reset Scores");
		layout.setConstraints(button, constraints);
		subpanel.add(button);

		constraints.ipadx = 50;
		constraints.weightx = 0.0;

		button = new Button("OK");
		layout.setConstraints(button, constraints);
		subpanel.add(button);

		pack();

		// Create the dialog for the new winners

		newBestTimeDialog = new NewBestTimeDialog((Frame) getParent(), this);

	}

	private void loadTimes() {

		DataInputStream stream;
		int i;

		try {

			stream = new DataInputStream(new FileInputStream("defaults/ms_score"));

			try{
				for(i = 0; i < TYPES; i++) {
					time[i]= stream.readInt();
					name[i] = stream.readUTF();
				}
			} catch (IOException e) {
				stream.close();
				throw(e);
			}

			stream.close();

		} catch (IOException e) {
			for(i = 0; i < TYPES; i++) {
				time[i] = DEFAULT_TIME;
				name[i] = DEFAULT_NAME;
			}
		}

		// Updates the corresponding labels on the dialog

		for(i = 0; i < TYPES; i++) {
			timeLabel[i].setText(time[i] + " seconds");
			nameLabel[i].setText(name[i]);
			timeLabel[i].invalidate();
			nameLabel[i].invalidate();
		}

		pack();
	}

	private void saveTimes() {

		DataOutputStream stream;
		int i;

		try {

			stream = new DataOutputStream(new FileOutputStream("defaults/ms_score"));

			try{
				for(i = 0; i < TYPES; i++) {
					stream.writeInt(time[i]);
					stream.writeUTF(name[i]);
				}
			} catch (IOException e) {}

			stream.close();

		} catch (IOException e) {}
	}

	private void resetTimes() {

		int i;

		for(i = 0; i < TYPES; i++) {
			time[i] = DEFAULT_TIME;
			name[i] = DEFAULT_NAME;
		}

		saveTimes();

	}

	void notifyWin(int user, int time) {

		// If it's not a type of user of which we keep record, do nothing

		if((user < 0) || (user >= TYPES))
			return;

		// If the score is not higher than our records, likewise

		if(this.time[user] <= time)
			return;

		// Call the NewBestTime dialog; it will get back
		// to us later

		newBestTimeDialog.show(user, time, DEFAULT_NAME);

	}

	void setTime(int user, int time, String name) {

		// If it's not a type of user of which we keep record, do nothing

		if((user < 0) || (user >= TYPES))
			return;


		this.time[user] = time;
		this.name[user] = name;

		// Save the score file and update the dialog

		saveTimes();
		loadTimes();
	}

	public boolean action(Event event, Object arg) {
		if(event.target instanceof Button)
			if(arg == "OK") {
				hide();
				return true;
			} else if(arg == "Reset Scores") {
				resetTimes();
				loadTimes();
				return true;
			} else
				return false;
		else
			return false;

	}

	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY) {
			hide();
			return true;
		} else
			return super.handleEvent(event);
	}
}
