/*

	Class: Board
	Description: Object representing/storing the state of the board.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

class Board {
	private BoardCell[][] cell;			    // cell grid
	private BoardProperties properties;	    // properties of the board

	private int opened;					    // cells opened so far
	private int flagged;					// cells currently flagged
	private boolean complete, lost;			// end of game flags
	private boolean modified;				// fair-play flag

	boolean marks;							// are marks allowed?

	public Board() {

		// Create and initialize the cell grid

		BoardProperties max_properties = BoardProperties.getMaxProperties();
		int i, j;

		cell = new BoardCell[max_properties.rows][max_properties.columns];

		for(i = 0; i < max_properties.rows; i++)
			for(j = 0; j < max_properties.columns; j++) {

				cell[i][j] = new BoardCell(this, i, j);

				if(i > 0) {
					cell[i][j].nhood.position[CellNeighborhood.NORTH] = cell[i-1][j];
					cell[i-1][j].nhood.position[CellNeighborhood.SOUTH] = cell[i][j];
				}
				if(j > 0) {
					cell[i][j].nhood.position[CellNeighborhood.WEST] = cell[i][j-1];
					cell[i][j-1].nhood.position[CellNeighborhood.EAST] = cell[i][j];
				}
				if(i > 0 && j > 0) {
					cell[i][j].nhood.position[CellNeighborhood.NORTHWEST] = cell[i-1][j-1];
					cell[i-1][j-1].nhood.position[CellNeighborhood.SOUTHEAST] = cell[i][j];
				}
				if(i > 0 && j < max_properties.columns-1) {
					cell[i][j].nhood.position[CellNeighborhood.SOUTHWEST] = cell[i-1][j+1];
					cell[i-1][j+1].nhood.position[CellNeighborhood.NORTHEAST] = cell[i][j];
				}
			}

		properties = null;

	}

	void setup(BoardProperties properties) {

		int i, j;

		if(this.properties != null) {

			// Resets all the cells modified during the last game

			for(i = 0; i < this.properties.rows; i++)
				for(j = 0; j < this.properties.columns; j++)
					cell[i][j].reset();
		}

		this.properties = properties;

		int n;

		for(n = 0; n < properties.mines; ) {

			i = (int) (Math.random() * properties.rows);
			j = (int) (Math.random() * properties.columns);

			// Fair-play: the first opened cell must not be a bomb

			if((i == 0) && (j == 0))
				continue;

			// If there isn't a bomb in there already, put one

			if(!cell[i][j].bomb) {
				cell[i][j].bomb = true;
				n++;
			}
		}

		// Initializes all the status variables

		opened = flagged = 0;
		complete = lost = modified = false;

	}

	BoardProperties getProperties() {
		return properties;
	}

	boolean isComplete() {
		return complete;
	}

	boolean hasLost() {
		return lost;
	}

	boolean isModified() {
		return modified;
	}

	void setCellViewer(int row, int column, BoardCellViewer viewer) {
		cell[row][column].viewer = viewer;
	}

	void openCell(int row, int column, boolean wideMode) {

		// If it is outside bounds, do nothing

		if((row < 0) || (row >= properties.rows)
		 ||(column < 0) || (column >= properties.columns))
			return;

		if(wideMode) {

			// If the cell isn't open already, do nothing

			if(!cell[row][column].opened)
				return;

			// If the number of flags equals the number of bombs, open the area

			CellNeighborhood nhood = cell[row][column].nhood;

			if(nhood.bombCount() == nhood.flagCount())
				nhood.open();

		} else {

			// If the cell is flagged, do nothing

			if(cell[row][column].mark == BoardCell.FLAG)
				return;

			// Fair-play: the first selected cell is never a bomb

			if(!modified && cell[row][column].bomb) {
				cell[0][0].bomb = true;
				cell[row][column].bomb = false;
			}
			modified = true;

			// Open the cell

			cell[row][column].open();
		}

		if(!lost && ((opened + properties.mines) == (properties.rows*properties.columns))) {
			complete = true;
			lost = false;
			flagAllBombs();
		}
	}

	private void openAllBombs() {

		int i, j;

		for(i = 0; i < properties.rows; i++)
			for(j = 0; j < properties.columns; j++)
				if(cell[i][j].bomb != (cell[i][j].mark == BoardCell.FLAG))
					cell[i][j].open();
	}

	void notifyOpenCell(int row, int column) {

		// If it's over, don't bother doing anything

		if(complete)
			return;

		// Increase the counter of opened cells

		opened++;

		// Did we loose?

		if(cell[row][column].bomb) {
			complete = true;
			lost = true;
			openAllBombs();
		}
	}

	void selectCell(int row, int column, boolean selected, boolean wideMode) {

		// If it is outside bounds, do nothing

		if((row < 0) || (row >= properties.rows)
		 ||(column < 0) || (column >= properties.columns))
			return;

		if(wideMode)
			cell[row][column].nhood.select(selected);
		else
			cell[row][column].select(selected);
	}

	void markCell(int row, int column) {

		// If it is outside bounds, do nothing

		if((row < 0) || (row >= properties.rows)
		 ||(column < 0) || (column >= properties.columns))
			return;

		// If it's opened, do nothing

		if(cell[row][column].opened)
			return;

		// Are we removing a flag?

		if(cell[row][column].mark == BoardCell.FLAG)
			flagged--;

		// Rotate cyclically the cell's marking status

		cell[row][column].mark++;
		cell[row][column].mark %= (marks ? 3 : 2);
		cell[row][column].redraw();

		// Are we adding a flag?

		if(cell[row][column].mark == BoardCell.FLAG)
			flagged++;
	}

	private void flagAllBombs() {

		int i, j;

		for(i = 0; i < properties.rows; i++)
			for(j = 0; j < properties.columns; j++)
				if(cell[i][j].bomb && cell[i][j].mark != BoardCell.FLAG) {
					cell[i][j].mark = BoardCell.FLAG;
					cell[i][j].redraw();
					flagged++;
				}
	}

	int bombCount() {
		return(properties.mines - flagged);
	}

	void redraw() {

		BoardProperties max_properties = BoardProperties.getMaxProperties();

		int i, j;

		for(i = 0; i < max_properties.rows; i++)
			for(j = 0; j < max_properties.columns; j++)
				cell[i][j].redraw();
	}
}
