/*

	Class: BoardCell
	Description: Object representing/storing the state of a singe cell.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

class BoardCell {
	static final int FLAG = 1;
	static final int QUESTION = 2;

	private Board board;						// board the cell belongs to
	private int row, column;					// coordinates of the cell

	CellNeighborhood nhood;						// array of neighboring cells
	boolean bomb, opened, selected, trigger;	// state of the cell
	int mark;									// cell marking

	BoardCellViewer viewer;						// visualize the cell

	public BoardCell(Board board, int row, int column) {

		// Store the coordinates and a pointer to the board

		this.board = board;
		this.row = row;
		this.column = column;

		// Create the new cell neighborhood

		nhood = new CellNeighborhood(this);

		// Initialize the state of the cell

		bomb = opened = selected = trigger = false;
		mark = 0;

		viewer = null;
	}

	void open() {

		// If it has already been opened or it is a marked bomb, do nothing

		if(opened || ((mark == FLAG) && bomb))
			return;

		// If it is out of bounds, likewise

		BoardProperties properties = board.getProperties();

		if((row >= properties.rows) || (column >= properties.columns))
			return;

		// Deselect (if selected) and open the cell

		opened = true;
		selected = false;
		if(mark == QUESTION)
			mark = 0;

		// Smart open: if it safe, open the neighboring cells

		if(!bomb && (nhood.bombCount() == 0))
			nhood.open();

		// Notify the board and the viewer of the change

		boolean lost = board.hasLost();
		board.notifyOpenCell(row, column);
		trigger = bomb && (lost != board.hasLost());
		redraw();
	}

	void select(boolean selected) {

		// If the call is redundant, return

		if(this.selected == selected)
			return;

		// Select the cell and notify the viewer

		this.selected = selected;
		redraw();
	}

	void reset() {
		if(bomb || opened || selected || trigger || (mark > 0)) {

			// Reset and redraw the cell

			bomb = opened = selected = trigger = false;
			mark = 0;

			redraw();
		}
	}

	void redraw() {

		if(viewer == null)
			return;

		int state;

		if(opened)
			if(bomb)
				state = trigger ? BoardCellViewer.TRIGGER : BoardCellViewer.BOMB;
			else
				state = (mark == FLAG) ? BoardCellViewer.CROSSED : nhood.bombCount();
		else switch(mark) {
			case FLAG:
				state = BoardCellViewer.FLAGGED;
				break;
			case QUESTION:
				state = selected ? BoardCellViewer.UNKNOWN_SELECTED : BoardCellViewer.UNKNOWN;
				break;
			default:
				state = selected ? BoardCellViewer.CLOSED_SELECTED : BoardCellViewer.CLOSED;
				break;
		}

		viewer.draw(row, column, state);
	}
}
