/*

	Class: BoardView
	Description: Component visualizing/controlling the board.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

import java.awt.*;

class BoardView extends Canvas implements MultipleColorModes, BoardCellViewer {
	private static final int CELL_SIZE = 16;	// default size of a cell

	private MineSweeper parent;
	private Board board;

	private boolean pressed;					// is the mouse being pressed on a cell?
	private int row, column;					// coordinates of the cell being pressed
	private boolean markMode, wideMode;
	private int color_mode;

	private static Image cell[][] = new Image[COLOR_MODES][9];
	private static Image cell_special[][] = new Image[COLOR_MODES][8];

	private Image offImage[] = new Image[COLOR_MODES];
	// implements double buffering for better graphics performance

	public BoardView(MineSweeper parent, int color_mode) {

		this.parent = parent;
		this.color_mode = color_mode;

		// Create the board

		board = new Board();

		BoardProperties max_properties = BoardProperties.getMaxProperties();
		int i, j;

		for(i = 0; i < max_properties.rows; i++)
			for(j = 0; j < max_properties.columns; j++)
				board.setCellViewer(i, j, this);

		// Load the symbols for the cell states

		parent.loadImages(cell, "images/ms_celld.gif");
		parent.loadImages(cell_special, "images/ms_cells.gif");
	}

	public BoardView(MineSweeper parent) {
		this(parent, MODE_BW);
	}

	void setup(BoardProperties properties) {

		// Initialize the board

		board.setup(properties);
		repaint();

		// Initialize the clock, counter and "happy face" button

		parent.initClock();
		parent.setBombCounter(board.bombCount());
		parent.setFaceButton(MineSweeper.HAPPY);

		// No cell selected initially

		pressed = false;
		row = column = -1;
		markMode = wideMode = false;
	}

	Board getBoard() {
		return board;
	}

	public int getColorMode() {
		return color_mode;
	}

	public void setColorMode(int mode) {
		if(mode != color_mode) {
			color_mode = mode;
			repaint();
		}
	}

	public boolean mouseDown(Event e, int x, int y) {

		// If the game has ended, do nothing

		if(board.isComplete())
			return true;

		pressed = true;

		// Determine the mode

		markMode = e.metaDown();
		wideMode = e.shiftDown();

		// Determine the cell being pressed

		row = y / CELL_SIZE;
		column = x / CELL_SIZE;

		// Redraw just the cell being pressed according to the mode

		if(markMode) {
			board.markCell(row, column);
			parent.setBombCounter(board.bombCount());
		} else {
			board.selectCell(row, column, true, wideMode);
			parent.setFaceButton(MineSweeper.WORRIED);
		}
		repaint();

		return true;
	}

	public boolean mouseDrag(Event e, int x, int y) {

		// If in marking mode (right click) or the game has ended do nothing

		if(markMode || board.isComplete())
			return true;

		// Determine if there is a cell being pressed

		Rectangle bounds = bounds();

		boolean pressed = bounds.inside(bounds.x + x, bounds.y + y);

		// If so, compute the coordinates

		BoardProperties properties = board.getProperties();
		int row, column;

		if(pressed) {
			row = y / CELL_SIZE;
			column = x / CELL_SIZE;
		} else
			row = column = -1;

		// Is it the same cell as before? If so, do nothing

		if((this.pressed == pressed) &&
		   ((pressed && (this.row == row)
					 && (this.column == column)) || !pressed))
		   return true;

		// Redraw the relevant cells

		if(this.pressed)
			board.selectCell(this.row, this.column, false, wideMode);
		if(pressed)
			board.selectCell(row, column, true, wideMode);

		repaint();

		// Change (if necessary) the face button

		if(this.pressed != pressed)
			parent.setFaceButton(pressed ? MineSweeper.WORRIED : MineSweeper.HAPPY);

		// Memorize the new selected cell location

		this.pressed = pressed;
		this.row = row;
		this.column = column;

		return true;
	}

	public boolean mouseUp(Event e, int x, int y) {
		if(pressed && !markMode) {

			pressed = false;

			// Check if the board is being modified for the first time

			boolean modified = board.isModified();

			// Open the selected cell

			board.selectCell(row, column, false, wideMode);
			board.openCell(row, column, wideMode);

			// Start the clock when if first modifying the board

			if(!modified && board.isModified())
				parent.startClock();

			row = column = -1;

			// Did we finish the game?

			if(board.isComplete()) {
				parent.notifyComplete(!board.hasLost());
				parent.setBombCounter(board.bombCount());
				parent.setFaceButton(board.hasLost() ? MineSweeper.SAD : MineSweeper.COOL);
			} else
				parent.setFaceButton(MineSweeper.HAPPY);

			repaint();
		}

		// Reset the modes

		markMode = wideMode = false;

		return true;
	}

	public void draw(int i, int j, int state) {
		if(offImage[color_mode] != null) {

			// Draw the content

			Graphics offGraphics;
			Image pict;
			int m;

			for(m = 0; m < COLOR_MODES; m++) {

				pict =(state >= 0) ? cell[m][state] : cell_special[m][-state-1];
				offGraphics = offImage[m].getGraphics();

				offGraphics.drawImage(pict, j*CELL_SIZE, i*CELL_SIZE, this);
			}
		}
	}

	public void paint(Graphics g) {
		update(g);
	}

	public void update(Graphics g) {
		if(offImage[color_mode] == null) {

			// Create the buffers

			BoardProperties max_properties = BoardProperties.getMaxProperties();
			int m;

			for(m = 0; m < COLOR_MODES; m++)
				offImage[m] =
					createImage(
						max_properties.columns*CELL_SIZE,
						max_properties.rows*CELL_SIZE);

			// Draw all cells in the newly created buffers

			board.redraw();
		}

		// Copies the off-line buffer

		BoardProperties properties = board.getProperties();

		g.clipRect(
			0, 0,
			properties.columns*CELL_SIZE,
			properties.rows * CELL_SIZE);
		g.drawImage(offImage[color_mode], 0, 0, this);
	}

	public Dimension preferredSize() {

		BoardProperties properties = board.getProperties();

 		return(
 			new Dimension(
 				properties.columns*CELL_SIZE,
 				properties.rows*CELL_SIZE));
	}

	public Dimension minimumSize() {

		BoardProperties properties = board.getProperties();

 		return(
 			new Dimension(
 				properties.columns*CELL_SIZE,
 				properties.rows*CELL_SIZE));
	}
}
