/*

	Class: CustomFieldDialog
	Description: Dialog to create a board of non-standard size.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

import java.awt.*;

final class CustomFieldDialog extends Dialog {
	private TextField heightField, widthField, minesField;
	private BoardProperties default_properties;

	public CustomFieldDialog(MineSweeper parent) {

		super(parent, "Custom Field", true);

		// Set up the dialog

		setLayout(new BorderLayout());
		setFont(new Font("Dialog", Font.PLAIN, 11));
		setResizable(false);

		// Set up the dialog content

		Panel panel, subpanel;
		GridBagLayout layout;
		GridBagConstraints constraints;
		Label label;
		Button button;

		layout = new GridBagLayout();
		setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.BOTH;
		constraints.insets = new Insets(10, 10, 10, 10);
		constraints.weightx = 1.0;
		constraints.weighty = 1.0;

		panel = new Panel();
		layout.setConstraints(panel, constraints);
		add(panel);
		panel.setLayout(new BorderLayout(10, 10));

		subpanel = new Panel();
		panel.add("Center", subpanel);

		subpanel.setLayout(new GridLayout(3, 2, 0, 15));

		label = new Label("Height:", Label.LEFT);
		subpanel.add(label);

		heightField = new TextField(8);
		subpanel.add(heightField);

		label = new Label("Width:", Label.LEFT);
		subpanel.add(label);

		widthField = new TextField(8);
		subpanel.add(widthField);

		label = new Label("Mines:", Label.LEFT);
		subpanel.add(label);

		minesField = new TextField(8);
		subpanel.add(minesField);

		// Add the dialog buttons

		subpanel = new Panel();
		panel.add("East", subpanel);

		layout = new GridBagLayout();
		subpanel.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.anchor = GridBagConstraints.NORTHEAST;
		constraints.insets = new Insets(0, 10, 4, 0);
		constraints.ipady = 2;
		constraints.weightx = 1.0;

		button = new Button("OK");
		layout.setConstraints(button, constraints);
		subpanel.add(button);

		constraints.gridheight = GridBagConstraints.REMAINDER;
		constraints.ipadx = 30;
		constraints.weighty = 1.0;

		button = new Button("Cancel");
		layout.setConstraints(button, constraints);
		subpanel.add(button);

		// Set default values for the properties

		setDefaultProperties(BoardProperties.getMinProperties());

		pack();

	}

	private BoardProperties getProperties() {

		// Returns the values indicated in the fields of the dialog

		BoardProperties properties = new BoardProperties(0, 0, 0);

		try {
			properties.rows = Integer.parseInt(heightField.getText());
		} catch (NumberFormatException e) {}
		try {
			properties.columns = Integer.parseInt(widthField.getText());
		} catch (NumberFormatException e) {}
		try {
			properties.mines = Integer.parseInt(minesField.getText());
		} catch (NumberFormatException e) {}

		properties.validate();

		return properties;

	}

	void setDefaultProperties(BoardProperties default_properties) {

		this.default_properties = default_properties;

		// Set the fields in the dialog

		heightField.setText(Integer.toString(default_properties.rows));
		widthField.setText(Integer.toString(default_properties.columns));
		minesField.setText(Integer.toString(default_properties.mines));
	}

	public boolean action(Event event, Object arg) {

		// Clicking on "OK" or "Cancel" close the dialog

		if(event.target instanceof Button) {

			MineSweeper parent = (MineSweeper) getParent();

			if(arg == "OK") {
				hide();
				parent.setupBoard(MineSweeper.CUSTOM, getProperties());
				return true;
			} else if(arg == "Cancel") {
				hide();
				parent.setupBoard(MineSweeper.CUSTOM, default_properties);
				return true;
			} else
				return false;
		} else
			return false;
	}

	public boolean handleEvent(Event event) {

		MineSweeper parent = (MineSweeper) getParent();

		if (event.id == Event.WINDOW_DESTROY) {
			hide();
			parent.setupBoard(MineSweeper.CUSTOM, default_properties);
			return true;
		} else
			return super.handleEvent(event);
	}
}

