/*

	Class: ImageButton
	Description: Button without text but with an (optional) picture.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

import java.awt.*;

public class ImageButton extends Canvas implements MultipleColorModes {
	private static final int BORDER_SIZE = 2;   // image inset

	private boolean pressed;		// is the button being pressed?
	private boolean inside;			// is the mouse inside the button area?
	private int color_mode;

	private Image image[] = new Image[COLOR_MODES];
	private Image image_pressed[] = new Image[COLOR_MODES];

	public ImageButton(Image image, Image image_pressed, int color_mode) {

		int m;

		for(m = 0; m < COLOR_MODES; m++) {
			this.image[m] = image;
			this.image_pressed[m] = image_pressed;
		}

		this.color_mode = color_mode;
		pressed = inside = false;
	}

	public ImageButton(Image image, Image image_pressed) {
		this(image, image_pressed, MODE_BW);
	}

	public ImageButton(Image image, int color_mode) {
		this(image, image, color_mode);
	}

	public ImageButton(Image image) {
		this(image, image);
	}

	public int getColorMode() {
		return color_mode;
	}

	public void setImage(Image image, Image image_pressed, int mode) {
		this.image[mode] = image;
		this.image_pressed[mode] = image_pressed;
		if(color_mode == mode)
			repaint();
	}

	public void setImage(Image image, int mode) {
		setImage(image, image, mode);
	}

	public void setColorMode(int mode) {
		if(mode != color_mode) {
			color_mode = mode;
			repaint();
		}
	}

	public boolean mouseDown(Event e, int x, int y) {
		pressed = inside = true;
		repaint();

		return super.mouseDown(e, x, y);
	}

	public boolean mouseEnter(Event e, int x, int y) {
		inside = true;
		if(pressed)
			repaint();

		return super.mouseEnter(e, x, y);
	}

	public boolean mouseExit(Event e, int x, int y) {
		inside = false;
		if(pressed)
			repaint();

		return super.mouseEnter(e, x, y);
	}

	public boolean mouseUp(Event e, int x, int y) {
		if(pressed) {
			pressed = false;
			if(inside) {
				repaint();

				// Generate an action event

				Event actionEvt = new Event(this, Event.ACTION_EVENT, this);
				postEvent(actionEvt);
			}
		}

		return super.mouseUp(e, x, y);
	}

	public void paint(Graphics g) {
	    update(g);
	}

	public void update(Graphics g) {

		Color background, top_left, bottom_right;

		switch(color_mode) {
			case MODE_BW:
				background = Color.lightGray;
				top_left = Color.white;
				bottom_right = Color.black;
				break;
			default:
				background = getBackground();
				top_left = background.brighter();
				bottom_right = background.darker();
				break;
		}

		if(pressed && inside) {
			Color temp = top_left;
			top_left = bottom_right;
			bottom_right = temp;
		}

		Dimension size = size();

		int i;

		for(i = 0; i < BORDER_SIZE; i++) {
			g.setColor(top_left);
			g.drawLine(i, i, i, i + (size.height-2*i));
			g.drawLine(i, i, i + (size.width-2*i), i);
			g.setColor(bottom_right);
			g.drawLine(i, i + (size.height-2*i), i + (size.width-2*i), i + (size.height-2*i));
			g.drawLine(i + (size.width-2*i), i, i + (size.width-2*i), i + (size.height-2*i));
		}
		g.setColor(background);
		g.fillRect(i, i, size.width-2*i, size.height-2*i);

		Image pict = pressed ? image_pressed[color_mode] : image[color_mode];

		// Draws the image centered

		int x = (size.width - pict.getWidth(this))/2;
		int y = (size.height - pict.getHeight(this))/2;

		g.drawImage(pict, x, y, this);
	}

	public Dimension minimumSize() {
		return(
			new Dimension(
				image[color_mode].getWidth(this) + 2*BORDER_SIZE,
				image[color_mode].getHeight(this) + 2*BORDER_SIZE));
	}

	public Dimension preferredSize() {
		return(
			new Dimension(
				image[color_mode].getWidth(this) + 2*BORDER_SIZE,
				image[color_mode].getHeight(this) + 2*BORDER_SIZE));
	}
}
