/*

	Class: LEDDisplay
	Description: Component visualizing a red LED display.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

import java.awt.*;

public class LEDDisplay extends Canvas implements MultipleColorModes {
	private static final int DIGIT_WIDTH = 13;   // default width of a digit
	private static final int DIGIT_HEIGHT = 23;  // default height of a digit

	private static final Color brightRed = new Color(255, 0, 0);
	private static final Color darkRed = new Color(95, 0, 0);

	// digits are represented as a sequence of seven segments
	private static final boolean ON[][] =
		{{true, true, true, true, true, true, false},
		 {false, false, false, false, true, true, false},
		 {true, false, true, true, false, true, true},
		 {true, false, false, true, true, true, true},
		 {false, true, false, false, true, true, true},
		 {true, true, false, true, true, false, true},
		 {true, true, true, true, true, false, true},
		 {true, false, false, false, true, true, false},
		 {true, true, true, true, true, true, true},
		 {true, true, false, true, true, true, true},
		 {false, false, false, false, false, false, true}};
	private static final int MINUS = 10;

	private int digits, value;
	private int color_mode;

	private Image offImage;

	public LEDDisplay(int value, int digits, int color_mode) {

		// Checks first if the value is valid

		digits = Math.max(digits, 1);
		value = Math.max(value, (int) -(Math.pow(10, digits-1)-1));
		value = Math.min(value, (int) Math.pow(10, digits)-1);

		this.digits = digits;
		this.value = value;
		this.color_mode = color_mode;
	}

	public LEDDisplay(int value, int digits) {
		this(0, digits, MODE_BW);
	}

	public LEDDisplay(int digits) {
		this(0, digits);
	}

	public int getValue() {
		return value;
	}

	public int getColorMode() {
		return color_mode;
	}

	public void setValue(int value) {

		// Checks first if the value is valid

		value = Math.max(value, (int) -(Math.pow(10, digits-1)-1));
		value = Math.min(value, (int) Math.pow(10, digits)-1);
		this.value = value;
		repaint();
	}

	public void setColorMode(int mode) {
		if(mode != color_mode) {
			color_mode = mode;
			repaint();
		}
	}

	public void paint(Graphics g) {
		update(g);
	}

	public void update(Graphics g) {

		Dimension size = size();

		// Updates the off-line buffer if necessary

		if((offImage == null)
		|| (offImage.getHeight(this) != size.height)
		|| (offImage.getWidth(this) != size.width))
			offImage = createImage(size.width, size.height);

		// Set the background color to black

		Graphics offGraphics = offImage.getGraphics();

		offGraphics.setColor(Color.black);
		offGraphics.fillRect(0, 0, size.width, size.height);

		int w = size.width / digits;	// width of a single digit
		int h = size.height;			// height of a single digit
		int xs[] = new int[7];		  // x and y coords of the vertices of
		int ys[] = new int[7];		  // the polygons forming the segments
		int i, n, d, x;

		Color on, off;

		switch(color_mode) {
			case MODE_BW:
				on = Color.white;
				off = Color.black;
				break;
			default:
				on = brightRed;
				off = darkRed;
				break;
		}

		for(i = 1, n = Math.abs(value); i <= digits; i++, n /= 10) {

			x = (digits-i)*w;	   // starting position
			d = n % 10;			 // current digit

			if((value < 0) && (i == digits))
				d = MINUS;		  // most significant digit of a negative number


			offGraphics.setColor(ON[d][0] ? on : off);
			xs[0] = x+2; xs[1] = x+w-2; xs[2] = x+w-5; xs[3] = x+5; xs[4] = x+2;
			ys[0] = 1; ys[1] = 1; ys[2] = 4; ys[3] = 4; ys[4] = 1;
			offGraphics.fillPolygon(xs, ys, 5);

			offGraphics.setColor(ON[d][1] ? on : off);
			xs[0] = x+1; xs[1] = x+1; xs[2] = x+4; xs[3] = x+4; xs[4] = x+1;
			ys[0] = 1; ys[1] = h/2-1; ys[2] = h/2-4; ys[3] = 4; ys[4] = 1;
			offGraphics.fillPolygon(xs, ys, 5);

			offGraphics.setColor(ON[d][2] ? on : off);
			xs[0] = x+1; xs[1] = x+1; xs[2] = x+4; xs[3] = x+4; xs[4] = x+1;
			ys[0] = h/2-1; ys[1] = h-2; ys[2] = h-5; ys[3] = h/2+2; ys[4] = h/2-1;
			offGraphics.fillPolygon(xs, ys, 5);

			offGraphics.setColor(ON[d][3] ? on : off);
			xs[0] = x+1; xs[1] = x+w-1; xs[2] = x+w-4; xs[3] = x+4; xs[4] = x+1;
			ys[0] = h-1; ys[1] = h-1; ys[2] = h-4; ys[3] = h-4; ys[4] = h-1;
			offGraphics.fillPolygon(xs, ys, 5);

			offGraphics.setColor(ON[d][4] ? on : off);
			xs[0] = x+w-1; xs[1] = x+w-1; xs[2] = x+w-4; xs[3] = x+w-4; xs[4] = x+w-1;
			ys[0] = h/2-1; ys[1] = h-2; ys[2] = h-5; ys[3] = h/2+2; ys[4] = h/2-1;
			offGraphics.fillPolygon(xs, ys, 5);

			offGraphics.setColor(ON[d][5] ? on : off);
			xs[0] = x+w-1; xs[1] = x+w-1; xs[2] = x+w-4; xs[3] = x+w-4; xs[4] = x+w-1;
			ys[0] = 1; ys[1] = h/2-1; ys[2] = h/2-4; ys[3] = 4; ys[4] = 1;
			offGraphics.fillPolygon(xs, ys, 5);

			offGraphics.setColor(ON[d][6] ? on : off);
			xs[0] = x+2; xs[1] = x+3; xs[2] = x+w-3; xs[3] = x+w-2; xs[4] = x+w-4; xs[5] = x+4; xs[6] = x+2;
			ys[0] = h/2-1; ys[1] = h/2-2; ys[2] = h/2-2; ys[3] = h/2-1; ys[4] = h/2+1; ys[5] = h/2+1; ys[6] = h/2-1;
			offGraphics.fillPolygon(xs, ys, 7);
		}

		g.drawImage(offImage, 0, 0, this);
	}

	public Dimension preferredSize() {
 		return(new Dimension(DIGIT_WIDTH*digits, DIGIT_HEIGHT));
	}

	public Dimension minimumSize() {
 		return(new Dimension(DIGIT_WIDTH*digits, DIGIT_HEIGHT));
	}
}
