/*

	Class: MineSweeper
	Description: MineSweeper application.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

public final class MineSweeper extends Frame implements MultipleColorModes {
	final static int BEGINNER = 0;	// user type
	final static int INTERMEDIATE = 1;
	final static int EXPERT = 2;
	final static int CUSTOM = 3;

	private int user;
	private BoardProperties properties;
	private boolean marks;					// flags for marks
	private int color_mode;

	static final int HAPPY = 0;		// face button pictures
	static final int SAD = 1;
	static final int WORRIED = 2;
	static final int COOL = 3;

	private static Image face[][] = new Image[COLOR_MODES][4];

	private Panel3D mainPanel;						// main panel of the game
	private BoardView boardView;					// board
	private LEDDisplay clock, counter;				// clock and counter displays
	private ImageButton faceButton;					// "happy face" button
	private ClockThread clockThread;				// asynchronously runs the clock

	private static final int USER_ITEM = 2;			// menu items for the type of user
	private static final int MARKS_ITEM = 7;		// menu item for marks ("?")

	private Menu gameMenu, helpMenu;

	private CustomFieldDialog customFieldDialog;	// dialogs
	private BestTimesDialog bestTimesDialog;
	private AboutDialog aboutDialog;

	public MineSweeper() {
		super("MineSweeper");

		setLayout(new BorderLayout());
		setResizable(false);
		setBackground(Color.lightGray);

		// Initialize game parameters

		loadAppDefaults();

		// Set up the menubar

		MenuBar menuBar = new MenuBar();
		setMenuBar(menuBar);

		gameMenu = new Menu("Game");
		menuBar.add(gameMenu);

		helpMenu = new Menu("Help");
		menuBar.add(helpMenu);
		menuBar.setHelpMenu(helpMenu);

		// Set up the main menu

		MenuItem menuItem;
		CheckboxMenuItem checkboxMenuItem;

		menuItem = new MenuItem("New");
		gameMenu.add(menuItem);

		gameMenu.add("-");

		checkboxMenuItem = new CheckboxMenuItem("Beginner");
		checkboxMenuItem.setState(user == BEGINNER);
		gameMenu.add(checkboxMenuItem);
		checkboxMenuItem = new CheckboxMenuItem("Intermediate");
		checkboxMenuItem.setState(user == INTERMEDIATE);
		gameMenu.add(checkboxMenuItem);
		checkboxMenuItem = new CheckboxMenuItem("Expert");
		checkboxMenuItem.setState(user == EXPERT);
		gameMenu.add(checkboxMenuItem);
		checkboxMenuItem = new CheckboxMenuItem("Custom...");
		checkboxMenuItem.setState(user == CUSTOM);
		gameMenu.add(checkboxMenuItem);

		gameMenu.add("-");

		checkboxMenuItem = new CheckboxMenuItem("Marks(?)");
		checkboxMenuItem.setState(marks);
		gameMenu.add(checkboxMenuItem);
		checkboxMenuItem = new CheckboxMenuItem("Color");
		checkboxMenuItem.setState(color_mode == MODE_COLOR);
		gameMenu.add(checkboxMenuItem);


		gameMenu.add("-");

		menuItem = new MenuItem("Best Times...");
		gameMenu.add(menuItem);

		gameMenu.add("-");

		menuItem = new MenuItem("Exit");
		gameMenu.add(menuItem);

		// Set up the help menu

		menuItem = new MenuItem("About MineSweeper");
		helpMenu.add(menuItem);

		// Load the faces of the "face" button

		loadImages(face, "images/ms_face.gif");

		// Create a new game board

		GridBagLayout layout;
		GridBagConstraints constraints;
		Panel3D panel1, panel2;

		mainPanel = new Panel3D(3, true, color_mode);
		mainPanel.setLayout(new BorderLayout());
		add("Center", mainPanel);
		add("South", new Label("MineSweeper 0.9e"));

		layout = new GridBagLayout();
		mainPanel.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.insets = new Insets(3 + 3, 3 + 3, 3, 3 + 3);
		constraints.weightx = 1.0;

		panel1 = new Panel3D(3, false, color_mode);
		layout.setConstraints(panel1, constraints);
		mainPanel.add(panel1);

		constraints.fill = GridBagConstraints.BOTH;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.gridheight = GridBagConstraints.REMAINDER;
		constraints.insets = new Insets(3, 3 + 3, 3 + 3, 3 + 3);
		constraints.weightx = 1.0;
		constraints.weighty = 1.0;

		panel2 = new Panel3D(3, false, color_mode);
		layout.setConstraints(panel2, constraints);
		mainPanel.add(panel2);

		panel2.setLayout(new BorderLayout());
		boardView = new BoardView(this, color_mode);
		panel2.add("Center", boardView);

		layout = new GridBagLayout();
		panel1.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.NONE;
		constraints.insets = new Insets(3, 3 + 3, 3, 0);
		constraints.weightx = 1.0;

		panel2 = new Panel3D(1, false, color_mode);
		layout.setConstraints(panel2, constraints);
		panel1.add(panel2);

		panel2.setLayout(new BorderLayout());
		counter = new LEDDisplay(0, 3, color_mode);
		panel2.add("Center", counter);

		constraints.insets = new Insets(3, 0, 3, 0);
		constraints.weightx = 0.0;

		panel2 = new Panel3D(1, false, color_mode);
		layout.setConstraints(panel2, constraints);
		panel1.add(panel2);

		panel2.setLayout(new BorderLayout());
		faceButton = new ImageButton(face[color_mode][HAPPY], color_mode);
		panel2.add("Center", faceButton);

		constraints.insets = new Insets(3, 0, 3, 3 + 1);
		constraints.anchor = GridBagConstraints.EAST;
		constraints.weightx = 1.0;

		panel2 = new Panel3D(1, false, color_mode);
		layout.setConstraints(panel2, constraints);
		panel1.add(panel2);

		panel2.setLayout(new BorderLayout());
		clock = new LEDDisplay(0, 3, color_mode);
		panel2.add("Center", clock);

		// Set up the board

		setupBoard(user, properties);

		// Set up the modal dialogs

		customFieldDialog = new CustomFieldDialog(this);
		bestTimesDialog = new BestTimesDialog(this);
		aboutDialog = new AboutDialog(this);
	}

	void setupBoard(int user, BoardProperties properties) {

		// If a previous game was running, we automatically lose it

		notifyComplete(false);

		// Check the appropriate menu item

		((CheckboxMenuItem) gameMenu.getItem(USER_ITEM + this.user)).setState(false);
		((CheckboxMenuItem) gameMenu.getItem(USER_ITEM + user)).setState(true);

		// Set the size of the new board according to the user type

		switch(user) {
			case BEGINNER:
				properties = new BoardProperties(8, 8, 10);
				break;
			case INTERMEDIATE:
				properties = new BoardProperties(16, 16, 40);
				break;
			case EXPERT:
				properties = new BoardProperties(16, 30, 99);
				break;
			default:
				user = CUSTOM;
				break;
		}

		// Set up the board

		boardView.setup(properties);
		boardView.getBoard().marks = marks;
		boardView.invalidate();
		pack();

		this.user = user;
		this.properties = properties;
	}

	void setupBoard() {
		setupBoard(user, properties);
	}

	void initClock() {
		clock.setValue(0);
	}

	void startClock() {

		// If another clock process is running, stop it and reset the counter

		if(clockThread != null) {
			clockThread.stop();
			initClock();
		}

		// Create and start the new clock process

		clockThread = new ClockThread(clock);
		clockThread.start();
	}

	void notifyComplete(boolean win) {

		// Stop the clock if it's still running

		if(clockThread != null)
			clockThread.stop();
		clockThread = null;

		// If we won, notify the high-score manager

		if(win)
			bestTimesDialog.notifyWin(user, clock.getValue());
	}

	void setBombCounter(int value) {
		counter.setValue(value);
	}

	void setFaceButton(int type) {

		int m;

		for(m = 0; m < COLOR_MODES; m++)
			faceButton.setImage(face[m][type], face[m][HAPPY], m);
	}

	void loadAppDefaults() {

		Properties defaults = new Properties();

		// Load the defaults from file

		FileInputStream defaultsStream;

		try{
			defaultsStream = new FileInputStream("defaults/ms_prefs");
			defaults.load(defaultsStream);
			defaultsStream.close();
		} catch (IOException e) {}

		// Extract the relevant parameters

		Integer ivalue, idefault;
		Boolean bvalue, bdefault;

		try {
			idefault = new Integer(BEGINNER);
			ivalue = new Integer(defaults.getProperty("user", idefault.toString()));
			user = ivalue.intValue();
		} catch (NumberFormatException e) {}

		properties = BoardProperties.getMinProperties();
		try {
			idefault = new Integer(properties.rows);
			ivalue = new Integer(defaults.getProperty("rows", idefault.toString()));
			properties.rows = ivalue.intValue();
		} catch (NumberFormatException e) {}
		try {
			idefault = new Integer(properties.columns);
			ivalue = new Integer(defaults.getProperty("columns", idefault.toString()));
			properties.columns = ivalue.intValue();
		} catch (NumberFormatException e) {}
		try {
			idefault = new Integer(properties.mines);
			ivalue = new Integer(defaults.getProperty("mines", idefault.toString()));
			properties.mines = ivalue.intValue();
		} catch (NumberFormatException e) {}
		properties.validate();

		bdefault = new Boolean(true);
		bvalue = new Boolean(defaults.getProperty("marks", bdefault.toString()));
		marks = bvalue.booleanValue();

		bdefault = new Boolean(true);
		bvalue = new Boolean(defaults.getProperty("color", bdefault.toString()));
		color_mode = bvalue.booleanValue() ? MODE_COLOR : MODE_BW;
	}

	private void saveAppDefaults() {

		Properties defaults = new Properties();

		// Store the relevant parameters

		Integer ivalue;
		Boolean bvalue;

		ivalue = new Integer(user);
		defaults.put("user", ivalue.toString());

		ivalue = new Integer(properties.rows);
		defaults.put("rows", ivalue.toString());
		ivalue = new Integer(properties.columns);
		defaults.put("columns", ivalue.toString());
		ivalue = new Integer(properties.mines);
		defaults.put("mines", ivalue.toString());

		bvalue = new Boolean(marks);
		defaults.put("marks", bvalue.toString());

		bvalue = new Boolean(color_mode == MODE_COLOR);
		defaults.put("color", bvalue.toString());

		// Save the defaults to file

		FileOutputStream defaultsStream;

		try{
			defaultsStream = new FileOutputStream("defaults/ms_prefs");
			defaults.save(defaultsStream, "- MineSweeper -");
			defaultsStream.close();
		} catch (IOException e) {}
	}

	void loadImages(Image image[][], String filename) {
		synchronized(image) {

			// If the first is loaded, assume all of the others are

			if(image[0][0] != null)
				return;

			Toolkit toolkit = Toolkit.getDefaultToolkit();
			MediaTracker tracker = new MediaTracker(this);
			Image pict = toolkit.getImage(filename);

			tracker.addImage(pict, 0);
			try {
				tracker.waitForID(0);
			} catch(InterruptedException e) {};

			int w = pict.getWidth(this) / COLOR_MODES;
			int h = pict.getHeight(this) / image[0].length;

			int m, i;

			for(m = 0; m < COLOR_MODES; m++)
				for(i = 0; i < image[m].length; i++) {

					int pix[] = new int[w*h];

					PixelGrabber grabber =
						new PixelGrabber(pict, m*w, i*h, w, h, pix, 0, w);
					try {
						grabber.grabPixels();
					} catch(InterruptedException e) {}

					image[m][i] =
						toolkit.createImage(
							new MemoryImageSource(w, h, pix, 0, w));
				}
		}
	}

	public int getColorMode() {
		return color_mode;
	}

	public void setColorMode(int mode) {
		if(mode != color_mode) {
			color_mode = mode;
			mainPanel.setColorMode(mode);
		}
	}

	public boolean action(Event event, Object arg) {
		if(event.target instanceof CheckboxMenuItem) {
			// A checkbox menu item has been activated

			String sourceLabel = ((CheckboxMenuItem) event.target).getLabel();
			boolean sourceState = ((CheckboxMenuItem) event.target).getState();

			if (sourceLabel == "Beginner") {
				setupBoard(BEGINNER, null);
				return true;
			} else  if (sourceLabel == "Intermediate") {
				setupBoard(INTERMEDIATE, null);
				return true;
			} else  if (sourceLabel == "Expert") {
				setupBoard(EXPERT, null);
				return true;
			} else  if (sourceLabel == "Custom...") {
				customFieldDialog.setDefaultProperties(properties);
				customFieldDialog.show();
				return true;
			} else  if (sourceLabel == "Marks(?)") {
				boardView.getBoard().marks = marks = sourceState;
				return true;
			} else  if (sourceLabel == "Color") {
				setColorMode(sourceState ? MODE_COLOR : MODE_BW);
				return true;
			} else
				return false;
		} else if(event.target instanceof MenuItem) {

			// A plain menu item has been activated

			String sourceLabel = ((MenuItem) event.target).getLabel();

			if(sourceLabel == "New") {
				setupBoard();
				return true;
			} else if (sourceLabel == "Best Times...") {
				bestTimesDialog.show();
				return true;
			} else  if (sourceLabel == "Exit") {
				saveAppDefaults();
				System.exit(0);
			} else  if (sourceLabel == "About MineSweeper") {
				aboutDialog.show();
				return true;
			}
		} else if(event.target instanceof ImageButton)
			if(arg == faceButton) {

				// The new game button has been pressed

				setupBoard();
				return true;
			}
		return false;
	}

	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY) {

			// Closing the main window exits the program

			saveAppDefaults();
			System.exit(0);
		}
		return super.handleEvent(event);
	}

	public static void main(String[] args) {

		MineSweeper mineSweeper = new MineSweeper();

		mineSweeper.show();
	}
}
