/*

	Class: MineSweeper
	Description: MineSweeper applet.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.minesweeper;

import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.net.*;

public class MineSweeper extends Applet implements MultipleColorModes {
	public final static int BEGINNER = 0;	// user type
	public final static int INTERMEDIATE = 1;
	public final static int EXPERT = 2;
	public final static int CUSTOM = 3;

	private int user;
	private BoardProperties properties;
	private boolean marks;					// flags for marks
	private int color_mode;

	public static final int HAPPY = 0;		// face button pictures
	public static final int SAD = 1;
	public static final int WORRIED = 2;
	public static final int COOL = 3;

	private static Image face[][] = new Image[COLOR_MODES][4];

	private Panel3D mainPanel;						// main panel of the game
	private BoardView boardView;					// board
	private LEDDisplay clock, counter;				// clock and counter displays
	private ImageButton faceButton;					// "happy face" button
	private ClockThread clockThread;				// asynchronously runs the clock

	public void init() {

		setLayout(new BorderLayout());
		setBackground(Color.lightGray);

		// Initialize game parameters

		this.loadAppDefaults();

    	// Load the faces of the "face" button

    	this.loadImages(face, "images/ms_face.gif");

		// Create a new game board

        GridBagLayout layout;
		GridBagConstraints constraints;
		Panel3D panel1, panel2;

        mainPanel = new Panel3D(3, true, color_mode);
        mainPanel.setLayout(new BorderLayout());
        add("Center", mainPanel);

		layout = new GridBagLayout();
		mainPanel.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.insets = new Insets(3 + 3, 3 + 3, 3, 3 + 3);
		constraints.weightx = 1.0;

        panel1 = new Panel3D(3, false, color_mode);
		layout.setConstraints(panel1, constraints);
        mainPanel.add(panel1);

		constraints.fill = GridBagConstraints.BOTH;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.gridheight = GridBagConstraints.REMAINDER;
		constraints.insets = new Insets(3, 3 + 3, 3 + 3, 3 + 3);
		constraints.weightx = 1.0;
		constraints.weighty = 1.0;

        panel2 = new Panel3D(3, false, color_mode);
		layout.setConstraints(panel2, constraints);
        mainPanel.add(panel2);

        panel2.setLayout(new BorderLayout());
		boardView = new BoardView(this, color_mode);
		panel2.add("Center", boardView);

		layout = new GridBagLayout();
		panel1.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.NONE;
		constraints.insets = new Insets(3, 3 + 3, 3, 0);
		constraints.weightx = 1.0;

        panel2 = new Panel3D(1, false, color_mode);
		layout.setConstraints(panel2, constraints);
        panel1.add(panel2);

        panel2.setLayout(new BorderLayout());
        counter = new LEDDisplay(0, 3, color_mode);
		panel2.add("Center", counter);

		constraints.insets = new Insets(3, 0, 3, 0);
		constraints.weightx = 0.0;

        panel2 = new Panel3D(1, false, color_mode);
		layout.setConstraints(panel2, constraints);
        panel1.add(panel2);

		panel2.setLayout(new BorderLayout());
		faceButton = new ImageButton(face[color_mode][HAPPY], color_mode);
		panel2.add("Center", faceButton);

		constraints.insets = new Insets(3, 0, 3, 3 + 1);
		constraints.anchor = GridBagConstraints.EAST;
		constraints.weightx = 1.0;

        panel2 = new Panel3D(1, false, color_mode);
		layout.setConstraints(panel2, constraints);
        panel1.add(panel2);

        panel2.setLayout(new BorderLayout());
        clock = new LEDDisplay(0, 3, color_mode);
		panel2.add("Center", clock);

		// Set up the board

		this.setupBoard(properties);

		validate();
    }

	protected void setupBoard(BoardProperties properties) {

        // If a previous game was running, we automatically lose it

        this.notifyComplete(false);

        // Set up the board

		boardView.setup(properties);
		boardView.getBoard().marks = marks;
        boardView.invalidate();

		this.properties = properties;
	}

    protected void setupBoard() {
        this.setupBoard(properties);
    }

    protected void initClock() {
		clock.setValue(0);
    }

    protected void startClock() {

        // If another clock process is running, stop it and reset the counter

		if(clockThread != null) {
            clockThread.stop();
		    this.initClock();
        }

        // Create and start the new clock process

        clockThread = new ClockThread(clock);
        clockThread.start();
    }

    protected void notifyComplete(boolean win) {

        // Stop the clock if it's still running

		if(clockThread != null)
            clockThread.stop();
        clockThread = null;
    }

    protected void setBombCounter(int value) {
        counter.setValue(value);
    }

	public void setFaceButton(int type) {

		int m;

		for(m = 0; m < COLOR_MODES; m++)
			faceButton.setImage(face[m][type], face[m][HAPPY], m);
	}

    private void loadAppDefaults() {

		// Set up the parameters

        Integer ivalue;
        Boolean bvalue;

        properties = BoardProperties.getMinProperties();
        if(getParameter("rows") != null)
            try {
                ivalue = new Integer(getParameter("rows"));
                properties.rows = ivalue.intValue();
            } catch (NumberFormatException e) {}
        if(getParameter("columns") != null)
            try {
                ivalue = new Integer(getParameter("columns"));
                properties.columns = ivalue.intValue();
            } catch (NumberFormatException e) {}
        if(getParameter("mines") != null)
            try {
                ivalue = new Integer(getParameter("mines"));
                properties.mines = ivalue.intValue();
            } catch (NumberFormatException e) {}
        properties.validate();

        marks = true;
        if(getParameter("marks") != null) {
            bvalue = new Boolean(getParameter("marks"));
            marks = bvalue.booleanValue();
        }

        color_mode = MODE_COLOR;
        if(getParameter("color") != null) {
            bvalue = new Boolean(getParameter("color"));
            color_mode = bvalue.booleanValue() ? MODE_COLOR : MODE_BW;
        }
    }

	protected void loadImages(Image image[][], String filename) {

		// If the first is loaded, assume all of the others are

		if(image[0][0] != null)
			return;

		Toolkit toolkit = Toolkit.getDefaultToolkit();
		MediaTracker tracker = new MediaTracker(this);
		Image pict = getImage(getCodeBase(), filename);

		tracker.addImage(pict, 0);
		try {
			tracker.waitForID(0);
		} catch(InterruptedException e) {};

		int w = pict.getWidth(this) / COLOR_MODES;
		int h = pict.getHeight(this) / image[0].length;

		int m, i;

		for(m = 0; m < COLOR_MODES; m++)
			for(i = 0; i < image[m].length; i++) {

				int pix[] = new int[w*h];

				PixelGrabber grabber =
					new PixelGrabber(pict, m*w, i*h, w, h, pix, 0, w);
				try {
					grabber.grabPixels();
				} catch(InterruptedException e) {}

				image[m][i] =
					toolkit.createImage(
						new MemoryImageSource(w, h, pix, 0, w));
			}
	}

	public int getColorMode() {
		return color_mode;
	}

	public void setColorMode(int mode) {
		if(mode != color_mode) {
			color_mode = mode;
			mainPanel.setColorMode(mode);
		}
	}

	public boolean action(Event e, Object arg) {
        if(e.target instanceof ImageButton)
		    if(arg == faceButton) {

		        // The new game button has been pressed

				this.setupBoard();
				return true;
		    }
		return false;
	}

	public String[][] getParameterInfo() {
	    String[][] info = {
	        {"rows", "Integer", "number of rows"},
	        {"columns", "Integer", "number of columns"},
	        {"mines", "Integer", "number of mines"},
	        {"marks", "Boolean", "marks (?) allowed/disallowed"},
	        {"color", "Boolean", "color or black/white"}
	    };
	    return info;
	}

	public String getAppletInfo() {
	    return "MineSweeper v0.9e written by Roberto Virga";
	}
}
